<?php

namespace Litecms\RankSettings\Providers;

use Illuminate\Support\ServiceProvider;

class RankSettingsServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'rank_settings');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'rank_settings');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerRankSettings();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerRankSettings()
    {
        $this->app->bind('rank_settings', function($app) {
            return new RankSettings($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('RankSettings', 'Lavalite\RankSettings\Facades\RankSettings');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('litecms.rank_settings', function ($app) {
            return $this->app->make('Litecms\RankSettings\RankSettings');
        });

                // Bind RankSetting to repository
        $this->app->bind(
            'Litecms\RankSettings\Interfaces\RankSettingRepositoryInterface',
            \Litecms\RankSettings\Repositories\Eloquent\RankSettingRepository::class
        );

        $this->app->register(\Litecms\RankSettings\Providers\AuthServiceProvider::class);
                $this->app->register(\Litecms\RankSettings\Providers\EventServiceProvider::class);
        
        $this->app->register(\Litecms\RankSettings\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and rank_settings's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/rank_settings.php', 'rank_settings'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeRankSettings::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['litecms.rank_settings'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('litecms/rank_settings.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/rank_settings')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/rank_settings')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
